/*
 *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "TypesManager.h"

#include "Debug.h"
#include "TypesManager_p.h"
#include "Macros_p.h"

using namespace GTLCore;

std::map< const GTLCore::Type*, const GTLCore::Type*> TypesManager::Private::knownArrays;
std::map< const GTLCore::Type*, std::map< int, const GTLCore::Type* > > TypesManager::Private::knownVectors;

STATIC_DELETER(DeleteTypesManager)
{
  TypesManager::Private::deleteStaticMaps();
}

TypesManager::TypesManager() : d(new Private)
{
}

TypesManager::~TypesManager()
{
  for( std::map< GTLCore::String, const GTLCore::Type*>::iterator it = d->knownTypes.begin(); it != d->knownTypes.end(); ++it)
  {
    delete it->second;
  }
  delete d;
}

const GTLCore::Type* TypesManager::getStructure( const GTLCore::String& _name) const
{
  std::map< GTLCore::String, const GTLCore::Type*>::const_iterator it = d->knownTypes.find(_name);
  if( it == d->knownTypes.end())
  {
    return 0;
  }
  return it->second;
}

const GTLCore::Type* TypesManager::getArray( const GTLCore::Type* _type)
{
  GTL_ASSERT( _type );
  std::map< const GTLCore::Type*, const GTLCore::Type*>::const_iterator it = TypesManager::Private::knownArrays.find( _type );
  if( it == TypesManager::Private::knownArrays.end())
  {
    GTLCore::Type* at = new GTLCore::Type( _type );
    TypesManager::Private::knownArrays[ _type] = at;
    return at;
  }
  return it->second;
}

const GTLCore::Type* TypesManager::getArray( const GTLCore::Type* _type, int _levelsCount)
{
  GTL_ASSERT( _type );
  const GTLCore::Type* type = _type;
  for(int i = 0; i < _levelsCount; ++i )
  {
    type = getArray( type );
  }
  return type;
}

const GTLCore::Type* TypesManager::getVector( const GTLCore::Type* _type, int _size )
{
  GTL_ASSERT( _type );
  GTL_ASSERT( _size > 0 );
  const GTLCore::Type*& type = TypesManager::Private::knownVectors[ _type ][ _size ];
  if( type == 0 )
  {
    type = new GTLCore::Type( _size, _type );
  }
  return type;
}
